/* -*- c -*-
 * Time-stamp: <2005-04-23 20:16:18 rsmith>
 * 
 * onepad/depad.c
 * Copyright (C) 2001--2005 R.F. Smith <rsmith@xs4all.nl>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "padfile.h"

#ifndef NULL
#define NULL (void*)0
#endif

#ifndef RETURN_IF_FAIL
#define RETURN_IF_FAIL(test) if (!(test)) return
#endif

#ifndef RETURN_VAL_IF_FAIL
#define RETURN_VAL_IF_FAIL(test,val) if (!(test)) return ((val))
#endif

#ifndef NDEBUG
/* __FUNCTION__ is a GCC feature. */
#undef debug
#define debug(TXT) fprintf(stderr,"%s %s(): %s\n",__FILE__,__FUNCTION__,TXT)
#else
#undef debug
#define debug(TXT) (void)0
#endif				/* NDEBUG */

/* This enables you to see which files have been used in building a
 * binary. N.B, __attribute is a GCC extension, so we test for __GNUC__. */
#ifdef __GNUC__
static char *SCMID __attribute__ ((unused)) = 
"$Id: depad.c 43 2005-04-23 18:19:09Z rsmith $";
#endif

extern void print_license(FILE * f);

int main(int argc, char *argv[])
{
	struct padfile *key, *cypher;
        FILE *cleartext;
        unsigned char in, out, keyb;
        int rv = 0;
	if (argc != 4) {
		printf("depad " VERSION
		       " Copyright (C) 2001--2005 by R.F. Smith "\
		       "<rsmith@xs4all.nl>\n");
		printf("Usage: depad keyfile cyphertext cleartext\n\n");
		print_license(stdout);
		return 0;
	}
	/* open the cyphertext file */
        cypher = pfopenr(argv[2]);
        if (cypher == NULL) {
                fprintf(stderr, "depad: error opening cyphertext file %s\n",
			argv[2]);
                return 1;
        }
	/* open the key file */
        key = pfopenr(argv[1]);
        if (key == NULL) {
                fprintf(stderr, "depad: error opening key file %s\n", argv[1]);
		pfclose(cypher);
                return 2;
        }
	/* check if the keys match */
	if (strcmp(cypher->id, key->id)) {
		fprintf(stderr, "depad: keys do not match\n");
		pfclose(cypher);
		pfclose(key);
		return 3;
	}
	/* open the cleartext file */
        cleartext = fopen(argv[3], "w");
        if (cleartext == NULL) {
                printf("enpad: unable to open cleartext file %s\n",
                       argv[3]);
		pfclose(cypher);
		pfclose(key);
		return 4;
        }
	/* decrypt the cyphertext, write cleartext */
	while (pfreadb(cypher, &in) == 0) {
		if (pfreadb(key, &keyb)) {
                        rv = 5;
                        fprintf(stderr, "depad: message clipped\n");
                        break;			
		}
		out = in ^ keyb;
		if (fwrite(&out, sizeof(unsigned char), 1, cleartext)!=1) {
                        rv = 6;
                        fprintf(stderr, "depad: error writing cleartext\n");
                        break;
		}
	}
	/* close the files */
        pfclose(key);
        pfclose(cypher);
        fclose(cleartext);
	return rv;
}
/* EOF depad.c */
