/* -*- c -*-
 * Time-stamp: <2005-04-23 20:17:45 rsmith>
 * 
 * onepad/genpad.c
 * Copyright (C) 2001--2005 R.F. Smith <rsmith@xs4all.nl>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <assert.h>
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "padfile.h"

#ifndef NULL
#define NULL (void*)0
#endif

#ifndef NDEBUG
/* __FUNCTION__ is a GCC feature. */
#undef debug
#define debug(TXT) fprintf(stderr,"%s %s(): %s\n",__FILE__,__FUNCTION__,TXT)
#else
#undef debug
#define debug(TXT) (void)0
#endif				/* NDEBUG */

/* This enables you to see which files have been used in building a
 * binary. N.B, __attribute is a GCC extension, so we test for __GNUC__. */
#ifdef __GNUC__
static char *SCMID __attribute__ ((unused)) = 
"$Id: genpad.c 43 2005-04-23 18:19:09Z rsmith $";
#endif

extern void print_license(FILE * f);

int main(int argc, char *argv[])
{
	unsigned char r;
	FILE *rf;
	char *buf;
	struct padfile *pf;
	long int keysize, cnt = 0;
	int rv = 0;

	/* Check argument and print help message if necessary. */
	if (argc < 3 || argc > 4) {
		printf("genpad " VERSION
		       " Copyright (C) 2001--2005 by R.F. Smith "\
		       "<rsmith@xs4all.nl>\n");
		printf("Usage: genpad id size [file]\n\n");
		print_license(stdout);
		exit(0);
	}

	/* create the key file */
	if (argc == 3) {
		if (asprintf(&buf, "%s.key", argv[1]) == -1) {
			fprintf(stderr, "genpad: cannot make keyfile name.\n");
			exit(2);
		}
		pf = pfopenw(buf, argv[1]);
		free(buf);
	} else {
		pf = pfopenw(argv[3], argv[1]);
	}
	if (pf == NULL) {
		fprintf(stderr, "genpad: cannot create the keyfile.\n");
		exit(2);
	}

	/* determine the size of the keyfile in kilobytes */
	keysize = strtol(argv[2], NULL, 10);
	if (keysize < 0) {
		fprintf(stderr, "genpad: key cannot have a negative size.\n");
		exit(3);
	}
	keysize *= 1024;
	
	/* open the random number device */
	rf = fopen("/dev/random", "r");
	if (rf == NULL) {
		fprintf(stderr,
			"genpad: cannot open the random number device.\n");
		exit(4);
	}

	/* generate the key */
	while (cnt < keysize) {
		if (fread(&r, sizeof(char), 1, rf) == 0) {
			fprintf(stderr,
				"genpad: cannot read random number.\n");
			rv = 5;
			break;
		}
		if (pfwriteb(pf, r)) {
			fprintf(stderr,
				"genpad: cannot write to keyfile.\n");
			rv = 6;
			break;
		}
		cnt++;
	}
	
	/* clean up */
	pfclose(pf);
	fclose(rf);

	return rv;
}

/* EOF genpad.c */
